#!/usr/bin/tclsh8.5

source simple-template.tcl

# Register the package
package provide CRUTEM-file-parser 1.0
package require Tcl                8.4

# Create the namespace
namespace eval ::CRUTEM-file-parser {
	# Export commands
	namespace export leapYearCheck parseCRUTEM
}


proc ::CRUTEM-file-parser::leapYearCheck {year} {
   # check if a leapyear

	set m4 [expr $year%4]
	set m100 [expr $year%100]
	set m400 [expr $year%400]
	set result [expr ($m4==0) && ($m100!=0) || ($m400==0)]
	return $result
}



proc ::addCrutemObs {start_year end_year xml paramlist} {

	set db [pg_connect -conninfo [list host = "tiger.dl.ac.uk" user = "remote-user" password = "Counter1" dbname = "acrid"]]

	puts "checking for existing crutem3 record"
	
	set origin_id "'crutem3'"
	

	set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $origin_id order by creation_date DESC limit 1"]
	set row [pg_result $result -llist]
	set ver_id 1
	
	if {[llength $row] > 0 } then  {
		set ver_id [expr {[lindex $row 0] + 1}]
		puts $ver_id
	}
	set title "'CRUTEM3 Gridded Temperature Series'"

	

    puts "adding CRUTEM3 observation with origin id: $origin_id and version: $ver_id ..."
	
	set result [pg_exec $db "INSERT INTO observations values ('$ver_id',$origin_id, $title, 'gridded-temperature-series', $start_year, $end_year, now(), 2, XMLPARSE (DOCUMENT '$xml'),2, 'GridSeries', NULL) "]
	if {[pg_result $result -status] != "PGRES_COMMAND_OK"} {
        	puts "[pg_result $result -error] adding CRUTEM observation"
	    	pg_result $result -clear
		
        } else {
		pg_result $result -clear
		foreach item $paramlist {
		
			if {$item != -1} {
			
				puts [string first "'staton-obs" $item]
			
				if {[string first "'station-obs" $item] == 0} then {

					pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, obs_value_origin, obs_value_version) values ($origin_id,'$ver_id', 'Station_Input_Files_for_all_process_steps', $item)"
				} elseif {[string first "'station-n" $item] == 0} then {

					pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, obs_value_origin, obs_value_version) values ($origin_id,'$ver_id', 'crutem3_gridding_process.conversion_to_anomalies.input', $item)"
				} elseif {[string first "'station-sd" $item] == 0} then {

					pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, obs_value_origin, obs_value_version) values ($origin_id,'$ver_id', 'crutem3_gridding_process.removal_of_outliers.input', $item)"
				}
			}
		}
	}
	pg_disconnect $db
}


proc ::CRUTEM-file-parser::parseCRUTEM {nfile template paramlist} {
	#open the file
	set infile [open $nfile r]

	set daysInMnth [split "-999,31,28,31,30,31,30,31,31,30,31,30,31" ","]
	set dayCount 15
	set total 15
	set counter 0
	set start_year 0
	set end_year 0

	while { [gets $infile line] >= 0 } {
    		
		# extract header - year/mnth
		if {[string length $line] < 70} {

			set year [string range $line 2 end-58]
			set month [string range $line 10 end-52]

			if {$counter == 0} {
				set first_time_point [string trim $year " "]
				set start_year $first_time_point
				append first_time_point [concat _[string trim $month " "]]
				append first_time_point "_1"
			}	

			set end_year [string trim $year " "]
			incr counter

			if {$month == 2 && [leapYearCheck $year]} {
				set daysToAdd 29 
			} else {
				set daysToAdd [lindex $daysInMnth $month] 
			}

		set total [expr {$daysToAdd + $total}]
		append dayCount " " $total

		} 
	}
	set latitude_list {-87.5 -82.5 -77.5 -72.5 -67.5 -62.5 -57.5 -52.5 -47.5 -42.5 -37.5 -32.5 -27.5 -22.5 -17.5 -12.5  -7.5  -2.5   2.5   7.5  12.5  17.5  22.5  27.5  32.5  37.5  42.5  47.5  52.5  57.5  62.5  67.5  72.5  77.5  82.5  87.5}
	set longitude_list {-177.5 -172.5 -167.5 -162.5 -157.5 -152.5 -147.5 -142.5 -137.5 -132.5 -127.5 -122.5 -117.5 -112.5 -107.5 -102.5  -97.5  -92.5  -87.5  -82.5  -77.5  -72.5  -67.5  -62.5  -57.5  -52.5  -47.5  -42.5  -37.5  -32.5  -27.5  -22.5  -17.5  -12.5   -7.5   -2.5    2.5    7.5   12.5   17.5   22.5   27.5   32.5   37.5   42.5   47.5   52.5   57.5   62.5   67.5   72.5   77.5   82.5   87.5   92.5   97.5  102.5  107.5  112.5  117.5  122.5  127.5  132.5  137.5  142.5  147.5  152.5  157.5  162.5  167.5  172.5  177.5}

	# open the template file
	set fh [open [concat $template/CRUTEM3-Result-Template.tmp] r]
	set code [simple-template::parse [read $fh] ]
	close $fh

	set xml [eval $code]

	
	addCrutemObs $start_year $end_year $xml $paramlist

	
}
