#!/usr/bin/env tclsh

source simple-template.tcl

# Register the package
package provide station-file-parser 1.0
package require Tcl                 8.4
package require Pgtcl

# Create the namespace
namespace eval ::station-file-parser {
	# Export commands
	namespace export trimSpace getSingleValue parseSingleNormal parseSingleSD parseSingleStation
}

proc ::station-file-parser::trimSpace {arg} {
   # function to remove extra spaces beween words

   set result [regexp -inline -all -- {\S+} $arg]
   return $result
}

proc ::station-file-parser::getSingleValue {arg key} {

    # reminder about id
    set arg [trimSpace $arg]
    set first [string first $key [string tolower $arg]]
    if {$first == 0} {
        set id [string first "=" $arg]
	set id [expr $id + 1]
	set station_id [string range $arg $id end]
	set station_id [string trim $station_id " " ]
	return $station_id
    } else {
	return -1
    }
}



proc ::get-db {arg1} {
	set db [pg_connect -conninfo [list host = "tiger.dl.ac.uk" user = "remote-user" password = "Counter1" dbname = "acrid"]]
	
	return $db
}

proc ::addStation {station_loc elevation country type station_id localid name db} {

	puts "adding new records for station $station_id to the database .."
	# adding a station record


	set result [pg_exec $db "INSERT INTO sampled_features values ('point($station_loc)',$elevation,'$country', '$type', '$station_id') "]
        if {[pg_result $result -status] != "PGRES_COMMAND_OK"} {
        	puts "[pg_result $result -error] adding station $station_id"
	    	pg_result $result -clear
		
        } else {
		

        	pg_result $result -clear
	
		#adding a station id record for the WMO ID
		pg_exec $db "INSERT INTO sf_identifiers (value, name, id_authority, sf_id) values ('$station_id', '$name', 1, '$station_id') "
		#adding a station id record for the local ID
		pg_exec $db "INSERT INTO sf_identifiers (value, name, id_authority, sf_id) values ('$localid', 'local id', 2, '$station_id') "
		puts "finished adding records for station $station_id to the database .."
	}

}


proc ::addStationObs {station_id station_loc template start end fry anders db} {

	puts "checking for existing station observation record"
	set origin_id [concat 'station-obs-$station_id']
	puts $origin_id

	#set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $origin_id order by creation_date DESC limit 1"]
	#set row [pg_result $result -llist]
	set ver_id 1
	
	#if {[llength $row] > 0 } then  {
	#	set ver_id [expr {[lindex $row 0] + 1}]
	#	puts $ver_id
	#}
	#set title [concat 'observation made at station $station_id']
	#puts "generating xml encoding for the observation result ..."

	#set fh [open $template r]
	#set code [simple-template::parse [read $fh] ]
	#close $fh
	#set xml [eval $code]
	
	#puts "done!"
    #    puts "adding station observation with origin id: $origin_id and version: $ver_id ..."
	#pg_exec $db "INSERT INTO observations values ('$ver_id',$origin_id, $title, 'temperature-series', $start, $end, now(), 2, XMLPARSE (DOCUMENT '$xml'),1, 'PointSeries', '$station_id') "
	#if {$anders > -1} {
	#	pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, other_value) values ($origin_id,'$ver_id', 'anders code', '$anders')"
	#}
	#pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, other_value, other_value_uom) values ($origin_id,'$ver_id', 'First Reliable Year', '$fry', 'year')"

	
	
	puts "done!"
	return "$origin_id,'$ver_id'"

}

proc ::addStationNormals {station_id template start_year end_year values db} {

	puts "checking for existing station normal record"
	set origin_id [concat 'station-n-$station_id']
	puts $origin_id

	#set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $origin_id order by creation_date DESC limit 1"]
	#set row [pg_result $result -llist]
	set ver_id 1
	
	#if {[llength $row] > 0 } then  {
	#	set ver_id [expr {[lindex $row 0] + 1}]
	#	puts $ver_id
	#}
	#set title [concat 'station normal for station $station_id']


	#set obs_value_origin [concat 'station-obs-$station_id']
	#set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $obs_value_origin order by creation_date DESC limit 1"]
	#set row [pg_result $result -llist]
	#set obs_value_version [lindex $row 0]


	#puts "generating xml encoding for the station normal $station_id ..."

	#set fh [open $template r]
	#set code [simple-template::parse [read $fh] ]
	#close $fh
	#set xml [eval $code]
	
	#puts "done!"


    #puts "adding station normal with origin id: $origin_id and version: $ver_id ..."

	#pg_exec $db "INSERT INTO observations values ('$ver_id',$origin_id, $title, 'station-normal', $start_year, $end_year, now(), 2, XMLPARSE (DOCUMENT '$xml'),3, 'PointSeries', '$station_id') "
	#pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, obs_value_origin, obs_value_version) values ($origin_id,'$ver_id', 'Calculation_of_Normals_SD.input', $obs_value_origin, $obs_value_version)"

	
	#puts "done!"
	return "$origin_id,'$ver_id'"

}

proc ::addStationSD {station_id template start_year end_year values db} {

	puts "checking for existing station observation record"
	set origin_id [concat 'station-sd-$station_id']
	puts $origin_id

	#set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $origin_id order by creation_date DESC limit 1"]
	#set row [pg_result $result -llist]
	set ver_id 1
	
	#if {[llength $row] > 0 } then  {
	#	set ver_id [expr {[lindex $row 0] + 1}]
	#	puts $ver_id
	#}
	#set title [concat 'standard deviations for station $station_id']


	#set obs_value_origin [concat 'station-obs-$station_id']
	#set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $obs_value_origin order by creation_date DESC limit 1"]
	#set row [pg_result $result -llist]
	#set obs_value_version [lindex $row 0]


	#puts "generating xml encoding for the station sd $station_id ..."

	#set fh [open $template r]
	#set code [simple-template::parse [read $fh] ]
	#close $fh
	#set xml [eval $code]
	
	#puts "done!"


    #puts "adding station sd with origin id: $origin_id and version: $ver_id ..."

	#pg_exec $db "INSERT INTO observations values ('$ver_id',$origin_id, $title, 'standard deviations', $start_year, $end_year, now(), 2, XMLPARSE (DOCUMENT '$xml'),3, 'PointSeries', '$station_id') "
	#pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, obs_value_origin, obs_value_version) values ($origin_id,'$ver_id', 'Calculation_of_Normals_SD.input', $obs_value_origin, $obs_value_version)"

	
	#puts "done!"
	return "$origin_id,'$ver_id'"

}

proc ::station-file-parser::parseSingleStation {nfile template type} {

	
	# db connection
        set db [get-db ""]

	#station variables
	
	set station_id -1
	set name -1           
	set country -1
	set localid -1
	set lat -1
	set lon -1
	set elevation -1
        set s_type "station"
        set exist 0
	
	variable start -1
	variable end -1
	variable fry -1
	variable anders -1
				
        variable nstart -1
	variable nend -1
	variable nvalues ""
	variable sstart -1
	variable send -1
	variable svalues ""
        
	set ctype $type
	
	if {$type == "all"} {
		set ctype "station"
	}




        #open the file
	set infile [open $nfile r]

        puts "parsing file $nfile..."

	while { [gets $infile line] >= 0 } {
      		

		set line [string trim [string tolower $line] " "]

    		if {$station_id == -1} {
			set station_id [getSingleValue $line "number"]
                        puts "checking if station $station_id already exists in the database"
			set result [pg_exec $db "SELECT count(*) FROM sampled_features where localid = '$station_id' "]
			set exist [pg_result $result -getTuple 0]
                        if {$exist < 1} {
				puts "station $station_id does not exist.."
			} else {
				puts "station $station_id  exists, skipping capturing station metadata"
			}

    		} elseif {$exist == 0 && $name == -1} {
			set name [getSingleValue $line "name"]
		} elseif {$exist == 0 && $country == -1} {
			set country [getSingleValue $line "country"]
		} elseif {$exist == 0 && $localid == -1} {
			set localid [getSingleValue $line "local id"]
		} elseif {$lat == -1} {
			set lat [getSingleValue $line "lat"]
		} elseif {$lon == -1} {
			set lon [getSingleValue $line "lon"]
		} elseif {$exist == 0 && $elevation == -1} {
			set elevation [getSingleValue $line "height"]
		} elseif {$ctype == "station"} {
			
			if {$start == -1} {
				
				set start [getSingleValue $line "start year"]
				
			} elseif {$end == -1} {
				set end [getSingleValue $line "end year"]
			} elseif {$fry == -1} {
				set fry [getSingleValue $line "first good year"]
			} elseif {$anders == -1} {
				
				set anders [getSingleValue $line "source id"]
				if {$type == "all"} {
					
					set ctype "normals"
				}
			} 

		} elseif {$ctype == "normals"} {
			
			#puts "get station normals metadata"


			if {$nstart == -1} {
        			# get the start year
        			set nstart [getSingleValue $line "normals source start year"]
    			} elseif {$nend == -1} {
        			# get the end year
        			set nend [getSingleValue $line "normals source end year"]
    			} elseif {$nvalues == ""} {
        			# get the values
        			set nvalues [getSingleValue $line "normals"]
				if {$type == "all"} {
					set ctype "sd"
				}
    			}

		} elseif {$ctype == "sd"} {
			
			#puts "get station sd metadata"

			if {$sstart == -1} {
        			# get the start year
        			set sstart [getSingleValue $line "standard deviations source start year"]
    			} elseif {$send == -1} {
        			# get the end year
        			set send [getSingleValue $line "standard deviations source end year"]
    			} elseif {$svalues == ""} {
        			# get the values
        			set svalues [getSingleValue $line "standard deviations"]
				set ctype "all"
    			} 
		} else {

			break
		}
		
	}
	
	puts "finished parsing file $nfile..."
	

	set station_loc [concat $lat $lon]

	close $infile
        
	# adding station record if needed

	if {$exist == 0} {
		addStation $station_loc $elevation $country $s_type $station_id $localid $name $db		
	}

	set station_param -1

	if {$type == "station" || $type == "all"} {
		

		lappend station_param [addStationObs $station_id $station_loc [concat $template/station.tmp] $start $end $fry $anders $db]
	}

	# only add normals and sds if they exist

	
	if {$type == "normals" || $type == "all" && $nstart > -1 } {
		# adding station normals
		lappend station_param [addStationNormals $station_id [concat $template/station-normal.tmp] $nstart $nend $nvalues $db]
	} else {
		set station_param -1
	}

	if {$type == "sd" || $type == "all" && $sstart > -1 } {
		# adding station normals
		lappend station_param [addStationSD $station_id [concat $template/station-sd.tmp] $sstart $send $svalues $db]
	} else {
		set station_param -1
	}
	


	
	pg_disconnect $db
	
	return $station_param
	
}
