#!/usr/bin/tclsh8.5

source simple-template.tcl

# Register the package
package provide tree-ring-raw-file-parser 1.0
package require Tcl                 8.4
package require Pgtcl



# Create the namespace
namespace eval ::tree-ring-raw-file-parser {
	# Export commands
	namespace export parseTreeRing addSite
}

proc ::connect-db {db host user password} {
	set con [pg_connect -conninfo [list host = $host user = $user password = $password dbname = $db]]
	
	return $con
}

proc ::tree-ring-raw-file-parser::addSite {station_loc elevation country type station_id} {

	set db [connect-db "acrid_tree_ring" "tiger.dl.ac.uk" "remote-user" "Counter1"]

	puts "adding new records for $type $station_id to the database .."
	# adding a station record


	set result [pg_exec $db "INSERT INTO sampled_features values ('point($station_loc)',$elevation,'$country', '$type', '$station_id') "]
        if {[pg_result $result -status] != "PGRES_COMMAND_OK"} {
        	puts "[pg_result $result -error] adding $type $station_id"
	    	pg_result $result -clear
		
        } else {
		

        	pg_result $result -clear
	
		#adding a station id record for the WMO ID
		pg_exec $db "INSERT INTO sf_identifiers (value, name, id_authority, sf_id) values ('$station_id', '$station_id', 5, '$station_id') "


		puts "finished adding records for $type $station_id to the database .."
	}
	pg_disconnect $db

}


proc ::addTree {type sf_id_db sf_id site_id {tree_id ""}} {

	set db [connect-db "acrid_tree_ring" "tiger.dl.ac.uk" "remote-user" "Counter1"]

	puts "adding new records for $type $sf_id to the database .."
	# adding a station record


	set result [pg_exec $db "INSERT INTO sampled_features values (NULL,NULL,NULL, '$type', '$sf_id_db') "]
        if {[pg_result $result -status] != "PGRES_COMMAND_OK"} {
        	puts "[pg_result $result -error] adding $type $sf_id_db"
	    	pg_result $result -clear
		
        } else {
		

        	pg_result $result -clear
	
		pg_exec $db "INSERT INTO sf_identifiers (value, name, id_authority, sf_id) values ('$sf_id', '$sf_id', 5, '$sf_id_db') "

		pg_exec $db "INSERT INTO related_sampled_features values ('related', '$sf_id_db', '$site_id') "

		if {$tree_id != ""} then {
			pg_exec $db "INSERT INTO related_sampled_features values ('sampled', '$sf_id_db', '$tree_id') "
		}


		puts "finished adding records for $type $sf_id to the database .."
	}
	pg_disconnect $db

}

proc ::getTreeCoreId {treeRingSiteIDCode} {

	set tcId ""
	set id [expr [string length $treeRingSiteIDCode] - 1]
	set last [string index $treeRingSiteIDCode $id]
	
	if {[regexp -all -nocase {[A-Z]} $last] > 0} then {
		set tcId [string range $treeRingSiteIDCode 0 [expr id - 1]]
	} else {
		set tcId [concat $treeRingSiteIDCode-core]
	}

	return $tcId
	

}

proc ::addTreeCoreObs {treeRingSiteIDCode template start end fileName} {

	# Observation Properties - edit/change as needed

	variable observedProperty "http://tridas.org/documents/xmldocs/1.2.2/tridas-1.2.20.html#id158"
	variable pithOffsetDistance "http://www.cru.uea.ac.uk/cru/projects/acrid/ontologies/tree-ring/pithOffsetDistance"
	variable pithOffsetRings "http://www.cru.uea.ac.uk/cru/projects/acrid/ontologies/tree-ring/pithOffsetRings"
	variable sapwoodRings "http://tridas.org/documents/xmldocs/1.2.2/tridas-1.2.20.html#id149"


	set db [connect-db "acrid_tree_ring" "tiger.dl.ac.uk" "remote-user" "Counter1"]

	puts "checking for existing tree-core observation record"
	puts $treeRingSiteIDCode
	set origin_id [concat '$treeRingSiteIDCode-raw']
	puts $origin_id

	set result [pg_exec $db "SELECT version_id FROM observations where origin_id = $origin_id order by creation_date DESC limit 1"]
	set row [pg_result $result -llist]
	set ver_id 1
	
	if {[llength $row] > 0 } then  {
		set ver_id [expr {[lindex $row 0] + 1}]
		puts $ver_id
	}
	set title [concat 'measurements taken from tree core $treeRingSiteIDCode']
	puts "generating xml encoding for the observation result ..."

	set fh [open $template r]
	set code [simple-template::parse [read $fh] ]
	close $fh
	set xml [eval $code]

	puts "done!"
        puts "adding tree-core observation with origin id: $origin_id and version: $ver_id ..."
	set result [pg_exec $db "INSERT INTO observations values ('$ver_id',$origin_id, $title, '$observedProperty', $start, $end, now(), 3, XMLPARSE (DOCUMENT '$xml'),1, 'PointSeries', '$treeRingSiteIDCode') "]
	puts "[pg_result $result -error]"
	    	pg_result $result -clear

	set result [pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, other_value, other_value_uom) values ($origin_id,'$ver_id', '$pithOffsetDistance', '-999', 'mm')"]
	puts "[pg_result $result -error]"
	    	pg_result $result -clear
	pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, other_value, other_value_uom) values ($origin_id,'$ver_id', '$pithOffsetRings', '-999', 'count')"
	pg_exec $db "INSERT INTO obs_params (ob_origin_id, ob_version_id, name, other_value, other_value_uom) values ($origin_id,'$ver_id', '$sapwoodRings', '-999', 'count')"

	
	
	puts "done!"
	pg_disconnect $db
	
	return "$origin_id,'$ver_id'"
	

}


proc ::tree-ring-raw-file-parser::parseTreeRing {nfile template siteName} {
	#open the file
	set infile [open $nfile r]

	set counter 0
    	set curID 0
	set curStart 0
	set curEnd 0
	set site_param -1
	puts $nfile
	
	while {1} {

		set line [gets $infile]
		
  		if {[eof $infile]} {  
			
			set tcId [getTreeCoreId $curID]
			puts "$curID $curStart $curEnd $tcId"
			addTree "tree" $curID $curID $siteName
			
			addTree "tree-core" $tcId $curID $siteName $curID
			lappend site_param [addTreeCoreObs $tcId $template $curStart $curEnd $nfile]

			close $infile
			break
		} else {		

		# extract header - year/mnth
			set sp_id [string first " " $line]

			set treeRingSiteIDCode [string range $line 0 [expr $sp_id - 1]]
		
			if {$curID !=0 && $curID != $treeRingSiteIDCode} then {
				
				set tcId [getTreeCoreId $curID]
				puts "Not End $curID $curStart $curEnd $tcId"
				addTree "tree" $curID $curID $siteName
			
				addTree "tree-core" $tcId $curID $siteName $curID
				
				lappend site_param [addTreeCoreObs $tcId $template $curStart $curEnd $nfile]
				set curStart 0
				set curEnd 0
				
			} 
			
			set curID $treeRingSiteIDCode
			set temp [string range $line [expr $sp_id + 1] end]
			set temp [string trim $temp " " ]
			
			set y_sp_id [string first " " $temp]
			

			set year [string range $temp 0 [expr $y_sp_id - 1]]
			
			if {$curStart == 0} then {
				set curStart $year
			} else {
				set curEnd $year
			}
			
			
			
			set linePrev $line
		} 
		
	}


	return $site_param
	
	# open the template file
	#set fh [open $template r]
	#set code [simple-template::parse [read $fh] ]
	#close $fh

  
	#eval $code

	# write the output to a file

	#set outfile [open [append treeRingSiteIDCode "-result.xml"] w]
	#puts $outfile $res
	#close $outfile
}
